/*
 * USB Driver for WebCams using the Ali M5602 and M5603c chipset.
 * 
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation, version 2.
 *
 */

#ifndef M560X_H_
#define M560X_H_

/* Driver info */
#define DRIVER_VERSION "0.1"
#define DRIVER_AUTHOR "Ali m560x Linux Driver Project <m560x@x3ng.com.br>"
#define DRIVER_DESC "Ali M560x Webcam Driver"

/* TODO this value must be inserted into linux/videodev.h */
#define VID_HARDWARE_M560X		40

/* Define these values to match your devices */
#define USB_M560X_VENDOR_ID			0x0402
#define USB_M560X_PRODUCT_M5602		0x5602
#define USB_M560X_PRODUCT_M5603c	0x5603

#define M560X_BYTES_PER_PIXEL   3
#define M560X_MAX_READ_SIZE     (512*242+128)
#define M560X_MAX_FRAME_SIZE    (M560X_BYTES_PER_PIXEL*320*240)
#define M560X_FRAMES            2

#define M560X_HEADER_SIZE       64



/* Structure to hold all of our device specific stuff */
struct m560x_camera {
	char					* name;
	
	struct semaphore		cam_lock;				/* guard against simultaneous accesses to the camera */
	unsigned int VideoMode;						/* 0x0100 = VGA, 0x0000 = CIF, 0x0300 = QVGA */
	int SupportedModes;
	int CIF;
	int VGA;
	int QVGA;
	int cwidth;             /* camera width */
	int cheight;            /* camera height */
	int maxwidth;           /* max video width */
	int maxheight;          /* max video height */
	int vwidth;             /* current width for video window */
	int vheight;            /* current height for video window */
 
	int is_initialized;
	
	struct video_device		* vdev;					/* the v4l video device for this device */
	
	struct usb_device		* udev;					/* the usb device for this camera */
	struct usb_interface	* interface;			/* the interface for this usb device */

	unsigned char 			* int_in_buffer;		/* the buffer to receive INT data */
	size_t					int_in_size;			/* the size of the INT receive buffer */
	__u8					int_in_endpointAddr;	/* the address of the INT in endpoint */

	unsigned char 			* isoc_in_buffer;		/* the buffer to receive ISOC data */
	size_t					isoc_in_size;			/* the size of the ISOC receive buffer */
	__u8					isoc_in_endpointAddr;	/* the address of the ISOC in endpoint */

	int						user;					/* user count for exclusive use */
	int						streaming;				/* Are we streaming video? */
	unsigned int			palette;

	int readcount;
	int framecount;
	int error;
	int dropped;
 
	struct semaphore		lock; 					/* to lock the structure */
	wait_queue_head_t		wq;						/* Processes waiting */
	struct kref				kref;
};
#define to_m560x_camera(d) container_of(d, struct m560x_camera, kref)


#define DBG_TRACE	0x10

#define PDEBUG(level, fmt, args...) \
	do { \
		if (debug & level)     \
			info("[%s:%d] " fmt, __FUNCTION__, __LINE__ , ## args); \
	} while (0)


#endif /*M560X_H_*/
